/*
The software is provided by the Institute of Commercial Cryptography Standards 
(ICCS), and is used for algorithm submissions in the Next-generation Commercial 
Cryptographic Algorithms Program. 

ICCS doesn't represent or warrant that the operation of the software will be 
uninterrupted or error-free in all cases. ICCS will take no responsibility for 
the use of the software or the results thereof, if the software is used for any 
other purposes.
*/

#ifndef KAT_KEM_H
#define KAT_KEM_H

// Set "OUTPUT_BLANK_TEST_VECTORS" as 0 to generate test vector files rather than blank template
#define OUTPUT_BLANK_TEST_VECTORS 1

// Set "ALGORITHM_INSTANCE" as your algorithm instance name (no more than 64 bytes)
#define ALGORITHM_INSTANCE "AlgorithmInstance"

#ifdef __cplusplus
extern "C"
{
#endif

	/// @brief Obtain the claimed byte length of the public key
	/// @return Claimed byte length of the public key
	unsigned long long kem_get_pk_byts();

	/// @brief Obtain the claimed byte length of the private key
	/// @return Claimed byte length of the private key
	unsigned long long kem_get_sk_byts();

	/// @brief Obtain the claimed byte length of the shared secret key
	/// @return Claimed byte length of the shared secret key
	unsigned long long kem_get_ss_byts();

	/// @brief Obtain the claimed byte length of the ciphertext
	/// @return Claimed byte length of the ciphertext
	unsigned long long kem_get_ct_byts();

	/// @brief Key generate
	/// @param pk Public key
	/// @param pk_byts Byte length of the public key
	/// @param sk Private key
	/// @param sk_byts Byte length of the private key
	/// @return If run successfully, return 0; otherwise, return -1 or a self-defined negative error code
	int kem_keygen(
		unsigned char *pk, unsigned long long *pk_byts,
		unsigned char *sk, unsigned long long *sk_byts);

	/// @brief Encapsulate
	/// @param pk Public key
	/// @param pk_byts Byte length of the public key
	/// @param ss Shared secret key
	/// @param ss_byts Byte length of the shared secret key
	/// @param ct Ciphertext
	/// @param ct_byts Byte length of the ciphertext
	/// @return If run successfully, return 0; otherwise, return -1 or a self-defined negative error code
	int kem_enc(
		unsigned char *pk, unsigned long long pk_byts,
		unsigned char *ss, unsigned long long *ss_byts,
		unsigned char *ct, unsigned long long *ct_byts);

	/// @brief Decapsulate
	/// @param sk Private key
	/// @param sk_byts Byte length of the private key
	/// @param ct Ciphertext
	/// @param ct_byts Byte length of the ciphertext
	/// @param ss Shared secret key
	/// @param ss_byts Byte length of the shared secret key
	/// @return If run successfully, return 0; otherwise, return -1 or a self-defined negative error code
	int kem_dec(
		unsigned char *sk, unsigned long long sk_byts,
		unsigned char *ct, unsigned long long ct_byts,
		unsigned char *ss, unsigned long long *ss_byts);

#ifdef __cplusplus
}
#endif
#endif