/*
The software is provided by the Institute of Commercial Cryptography Standards 
(ICCS), and is used for algorithm submissions in the Next-generation Commercial 
Cryptographic Algorithms Program. 

ICCS doesn't represent or warrant that the operation of the software will be 
uninterrupted or error-free in all cases. ICCS will take no responsibility for 
the use of the software or the results thereof, if the software is used for any 
other purposes.
*/

#ifndef KAT_SIG_H
#define KAT_SIG_H

// Set "OUTPUT_BLANK_TEST_VECTORS" as 0 to generate test vector files rather than blank template
#define OUTPUT_BLANK_TEST_VECTORS 1

// Set "ALGORITHM_INSTANCE" as your algorithm instance name (no more than 64 bytes)
#define ALGORITHM_INSTANCE "AlgorithmInstance"

#ifdef __cplusplus
extern "C"
{
#endif

	/// @brief Obtain the claimed byte length of the public key
	/// @return Claimed byte length of the public key
	unsigned long long sig_get_pk_byts();

	/// @brief Obtain the claimed byte length of the private key
	/// @return Claimed byte length of the private key
	unsigned long long sig_get_sk_byts();

	/// @brief Obtain the claimed byte length of the signature
	/// @return Claimed byte length of the signature
	unsigned long long sig_get_sn_byts();

	/// @brief Key generate
	/// @param pk Public key
	/// @param pk_byts Byte length of the public key
	/// @param sk Private key
	/// @param sk_byts Byte length of the private key
	/// @return If run successfully, return 0; otherwise, return -1 or a self-defined negative error code
	int sig_keygen(
		unsigned char *pk, unsigned long long *pk_byts,
		unsigned char *sk, unsigned long long *sk_byts);

	/// @brief Sign
	/// @param sk Private key
	/// @param sk_byts Byte length of the private key
	/// @param m Message
	/// @param m_byts Byte length of the message
	/// @param sn Signature
	/// @param sn_byts Byte length of the signature
	/// @return If run successfully, return 0; otherwise, return -1 or a self-defined negative error code
	int sig_sign(
		unsigned char *sk, unsigned long long sk_byts,
		unsigned char *m, unsigned long long m_byts,
		unsigned char *sn, unsigned long long *sn_byts);

	/// @brief Verify
	/// @param pk Public key
	/// @param pk_byts Byte length of the public key
	/// @param sn Signature
	/// @param sn_byts Byte length of the signature
	/// @param m Message
	/// @param m_byts Byte length of the message
	/// @return If run successfully, return 0; otherwise, return -1 or a self-defined negative error code
	int sig_verify(
		unsigned char *pk, unsigned long long pk_byts,
		unsigned char *sn, unsigned long long sn_byts,
		unsigned char *m, unsigned long long m_byts);

#ifdef __cplusplus
}
#endif
#endif