/*
The software is provided by the Institute of Commercial Cryptography Standards 
(ICCS), and is used for algorithm submissions in the Next-generation Commercial 
Cryptographic Algorithms Program. 

ICCS doesn't represent or warrant that the operation of the software will be 
uninterrupted or error-free in all cases. ICCS will take no responsibility for 
the use of the software or the results thereof, if the software is used for any 
other purposes.
*/

#ifndef DRNG_H
#define DRNG_H

#define SEEDLEN_BYTE (55)

#ifdef __cplusplus
extern "C"
{
#endif

    typedef struct
    {
        unsigned char V[SEEDLEN_BYTE];
        unsigned char C[SEEDLEN_BYTE];
        unsigned char reseed_counter[SEEDLEN_BYTE];
    } DRNG_ctx;

    /// @brief  Initialize Deterministic Random Number Generator (DRNG)
    ///         This function must be called before the first call of the get_random_number()
    /// @param drng Unique name of the DRNG
    /// @param seed The base address of input seed
    /// @param seed_length_Byte The valid BYTES of input seed
    /// @return 0 for success, others for error
    int init_random_number(DRNG_ctx *drng, const unsigned char *seed, unsigned long long seed_length_Byte);

    /// @brief Generate Pseudo Random Number
    /// @param drng Initialized DRNG
    /// @param random_number The base address to save the generated PRN
    /// @param random_number_length_bit The valid BITS of output PRN
    /// @return 0 for success, others for error
    int get_random_number(DRNG_ctx *drng, unsigned char *random_number, unsigned long long random_number_length_bit);

#ifdef __cplusplus
}
#endif
#endif